#function to calculate Hs, Ht and Gst from a matrix of allele frequencies of diallelic loci
#every row is a population, every column is a locus (one freq per locus)
#there is no sample size correction for the number of individuals
#but there is sample size correction for the number of populations
sum.stats = function(freqs, ploidies=2){
	#make sure that vector with ploidy levels is of correct length
	if(length(ploidies) == 1){
		ploidies = rep(ploidies, dim(freqs)[1])	
	}
	
	#define p and q
	p = freqs
	q = 1-p
	num = dim(p)[1]	
	
	#calculate Hs
	squared = (p^2 + q^2)			
	Hs = 1 - apply(squared, 2, mean)
	
	#calculate Ht
	overallP = apply(p, 2, mean)			
	overallQ = 1-overallP			
	Ht = 1 - (overallP^2 + overallQ^2)
	
	#take averages, only for loci with variation
	#note that this upwardly biases the estimates
	#but we are mostly interested in Fst anyway, which cannot be calculated when Ht==0
	Hs = mean(Hs[Ht>0])
	Ht = mean(Ht[Ht>0])
	
	#correct Ht for number of populations
	Dpst = (num / (num-1)) * (Ht - Hs)
	Hpt = Hs + Dpst
	#now calculate Gst with this correction
	Fst = (Hpt - Hs) / Hpt
	
	#prepare output
	return(c(Hs, Ht, Fst))
}


#set some parameters (default value)
num.pops = 2			# how many populations are simulated (2)
pop.size = 1000			# the number of individuals per population, all populations are the same size (1000)
mig.rate = 0.0001		# migration among populations, following island model (0.0001)
num.loci = 1000			# the number of loci, all loci are diallelic SNPs (1000)
mut.rate = 0.00001		# mutation rate per locus (0.00001)
num.gens = 10000		# the number of generations to run, for bigger populations, use more generations (10000)
ploidy = 2		 		# the ploidy level (2)

#define some colors
cols= c("olivedrab3", "firebrick3", "dodgerblue3", "sienna1", "darkorchid3", "gold1")

#make a pairwise migration matrix
#replacing this by a custom migration matrix allows using other migration models than the island model
mig.ratematrix = array(mig.rate / (num.pops-1), c(num.pops, num.pops))
for(p in 1:num.pops){
	mig.ratematrix[p,p] = 1-mig.rate
}

#expand ploidy level to a vector (for every population one)
ploidies = rep(ploidy, num.pops)	

#the populations are represented by a simple array of integers, initialised with maximum diversity
#(both alleles have a frequency of 0.5)
pops = NULL
for(p in 1:num.pops){
	counts = rbinom(num.loci, ploidies[p]*pop.size, 0.5)
	pops = rbind(pops, counts)
}

#already declare a variable in which we will store the results
allstats = NULL
			
#do generations
for(g in 1:num.gens){	
	#turn the counts of the allele into frequencies
	freqs = NULL
	for(p in 1:num.pops){
		freqs = rbind(freqs, pops[p,] / (ploidies[p]*pop.size))	
	}

	#get expectation of allele frequencies after migration
	exp = array(NA,dim(pops)) 
	for(p in 1:num.pops){
		exp[p,] = colSums(freqs*mig.ratematrix[,p])
	}
	
	#now some mutation
	exp = (1-mut.rate)*exp + mut.rate*(1-exp)

	#use the expected frequencies for random sampling from a binomial distribution				
	newpops = NULL
	for(p in 1:num.pops){
		counts = rbinom(num.loci, ploidies[p]*pop.size, exp[p,])
		newpops = rbind(newpops, counts)
	}
	pops = newpops
	
	#every 100 generations, calculate Hs, Ht, and Gst and store this
	if(g %% 100 == 0){
		print(g)
		
		freqs = NULL
		for(p in 1:num.pops){
			freqs = rbind(freqs, pops[p,] / (ploidies[p]*pop.size))	
		}
		allstats = rbind(allstats, c(g,sum.stats(freqs, ploidies)))	
	}	
}

#give the output some names so we know what we are looking at
colnames(allstats) = c("Generation", "Hs", "Ht", "Fst")

#create empty plot
plot(allstats[,1], allstats[,"Hs"], ylim=c(0,1), xlab="Generation", ylab="Statistic", pch=NA, font.lab=2)
#now add lines for summary stats
lines(allstats[,1], allstats[,"Hs"], col = cols[1], lwd=2)
lines(allstats[,1], allstats[,"Ht"], col = cols[2], lwd=2)
lines(allstats[,1], allstats[,"Fst"], col = cols[3], lwd=2)
#finally the legend
legend("topleft", c("Hs", "Ht", "Fst"), col=cols[1:4], lwd=2)

					

