### LOAD PACKAGES
library(vcfR)
library(adegenet)
library(adegraphics)

# also load the two custom-modified functions in a separate file
source(file="Part 4 custom functions.R")

#   VCF import, change to genlight, basic data inspection
### IMPORT SNP dat from VCF 
vcf <- read.vcfR("Arenosa_data.vcf.gz")   #read in all data
head(vcf)               #check the vcf object

### convert to genlight 
aa.genlight <- vcfR2genlight.tetra(vcf, n.cores=1)
locNames(aa.genlight) <- paste(vcf@fix[,1],vcf@fix[,2],sep="_")   # add real SNP.names
pop(aa.genlight)<-substr(indNames(aa.genlight),1,7)               # add pop names: here "population" (group) names are first 7 chars of ind name

# check the genlight
aa.genlight                        # check the basic info on the genlight object
indNames(aa.genlight)              # individual names
ploidy(aa.genlight)                # ploidy level inferred from the data
pop(aa.genlight)                   # population assignment

### get separate genlights per each pop.
aa.genlight.sep <- seppop(aa.genlight)   # separate genlight per population
aa.genlight.sep                          # list of genlights
names(aa.genlight.sep)                   # pop names




############
#   PCA on on real dataset
############

# first make a simple matrix from the genlight object
dips <-as.matrix(aa.genlight.sep$DIP_ALP)[,] 
tets <-as.matrix(aa.genlight.sep$TET_ALP)[,] 

#combine into single new object
combi = new("genlight", rbind(dips, tets))

#do the pca
pca.1 <- glPcaFast(combi, nf=4)             # for running this, firstly load the modified function at the end of this doc

# proportion of explained variance by first three axes
pca.1$eig[1]/sum(pca.1$eig) # proportion of variation explained by 1st axis
pca.1$eig[2]/sum(pca.1$eig) # proportion of variation explained by 2nd axis
pca.1$eig[3]/sum(pca.1$eig) # proportion of variation explained by 3rd axis

# ploidy - differentiated plots
g1 <- s.class(pca.1$scores, as.factor(as.vector(ploidy(combi))), xax=1, yax=2, col=transp(c("#FF0000", "#0000FF")), 
              ellipseSize=0, starSize=0, ppoints.cex=4, paxes.draw=T, pgrid.draw =F, plab.cex = 0 , plot = FALSE)
g2 <- s.label (pca.1$scores, xax=1, yax=2, ppoints.col = "red", plabels = list(box = list(draw = FALSE), 
                                                                               optim = TRUE), paxes.draw=T, pgrid.draw =F, plabels.cex=1, plot = FALSE)
ADEgS(c(g1, g2), layout = c(1, 2))





############
#   PCA with fully resolved randomly generated autotetraploids
############

#calculate allele frequencies
freqs = apply(dips, 2, mean, na.rm = TRUE)/2
num.tets = length(indNames(aa.genlight.sep$TET_ALP))
num.loci = dim(dips)[2]
rand.tets = NULL
for(i in 1:num.tets){
	rand.tets = rbind(rand.tets, rbinom(num.loci, 4, freqs))
}
row.names(rand.tets) = sprintf("tet_%d", 1:num.tets)


#make a new genlight object from diploids and tetraploids
combi.rand = new("genlight", rbind(dips, rand.tets))
pca.2 <- glPcaFast(combi.rand, nf=4) 

# proportion of explained variance by first three axes
pca.2$eig[1]/sum(pca.2$eig) # proportion of variation explained by 1st axis
pca.2$eig[2]/sum(pca.2$eig) # proportion of variation explained by 2nd axis
pca.2$eig[3]/sum(pca.2$eig) # proportion of variation explained by 3rd axis

# ploidy - differentiated plots
g1 <- s.class(pca.2$scores, as.factor(as.vector(ploidy(combi.rand))), xax=1, yax=2, col=transp(c("#FF0000", "#0000FF")), 
              ellipseSize=0, starSize=0, ppoints.cex=4, paxes.draw=T, pgrid.draw =F, plab.cex = 0 , plot = FALSE)
g2 <- s.label (pca.2$scores, xax=1, yax=2, ppoints.col = "red", plabels = list(box = list(draw = FALSE), 
                                                                               optim = TRUE), paxes.draw=T, pgrid.draw =F, plabels.cex=1, plot = FALSE)
ADEgS(c(g1, g2), layout = c(1, 2))







############
#   PCA on with randomly generated tetraploids without dosage info
############

#now remove the dosage
rand.tets.und = rand.tets
rand.tets.und[rand.tets.und == 1 | rand.tets.und == 3] =2

combi.und = new("genlight", rbind(dips, rand.tets.und))
pca.3 <- glPcaFast(combi.und, nf=4) 

# proportion of explained variance by first three axes
pca.3$eig[1]/sum(pca.3$eig) # proportion of variation explained by 1st axis
pca.3$eig[2]/sum(pca.3$eig) # proportion of variation explained by 2nd axis
pca.3$eig[3]/sum(pca.3$eig) # proportion of variation explained by 3rd axis

# ploidy - differentiated plots
g1 <- s.class(pca.3$scores, as.factor(as.vector(ploidy(combi.und))), xax=1, yax=2, col=transp(c("#FF0000", "#0000FF")), 
              ellipseSize=0, starSize=0, ppoints.cex=4, paxes.draw=T, pgrid.draw =F, plab.cex = 0 , plot = FALSE)
g2 <- s.label (pca.3$scores, xax=1, yax=2, ppoints.col = "red", plabels = list(box = list(draw = FALSE), 
                                                                               optim = TRUE), paxes.draw=T, pgrid.draw =F, plabels.cex=1, plot = FALSE)
ADEgS(c(g1, g2), layout = c(1, 2))

