#set some parameters
pop.size = 3000      # the total number of chromosome copies in each population, distributed over the two ploidy levels
				     # the default value of 3000 is equivalent to 500 diploid and 500 tetraploid individuals (2*500 + 4* 500 = 3000) 
mig.rate = 0.001     # the fraction of chromosome copies that migrates from one population to the other per generation
mut.rate = 0.00001   # the probability that an allele mutates to the other allele per generation
num.loci = 200       # the number of loci
num.pops = 2         # the number of populations, each carrying an equal number individuals of the two cytotypes
                     # mating within populations is completely random with respect to the cytotypes (form a single gene pool)
num.gens = 10000		 # the number of generations to run the model, should be long enough to reach equilibrium
sample.size = 50     # the number of individuals of each ploidy level in each population sampled for analysis in Structure
                     # keep this modest, unless you want a looooong coffee break
ploidy1 = 2			 # the ploidy level of the first cytotype
ploidy2 = 4			 # the ploidy level of the second cytotype
maxPloi = max(ploidy1,ploidy2)

#define some funky sounding colors for plotting, add more colors when using >6 populations
cols= c("olivedrab3", "firebrick3", "dodgerblue3", "gold1", "darkorchid3", "sienna1")					

#start with a frequency of 0.2 for every locus in every population
freqs = array(0.2, c(num.loci, num.pops))

#do the generations of mutation, migration and drift
for(g in 1:num.gens){
	#first calculate expected frequencies after mutation
	freqs_exp = freqs - mut.rate*freqs + mut.rate*(1-freqs)
	#now calculate expected frequencies after migration
	freqs_exp = freqs_exp - mig.rate*freqs_exp + mig.rate*colMeans(freqs_exp)
	#get new frequencies by drawing random numbers from a binomial distribution based on expected frequencies
	freqs=rbinom(length(freqs_exp),pop.size, freqs_exp)/pop.size
	dim(freqs) = c(num.loci, num.pops)
}
#now the simulation part of the script is finished, the rest is just taking care of the results

#calculate final summary statistics
Hs = mean(1 - rowMeans(freqs^2 + (1-freqs)^2))
Ht = mean(1 - (rowMeans(freqs)^2 + rowMeans(1-freqs)^2))
Fst = num.pops*(Ht-Hs)/(num.pops*Ht-Hs)

print(c(Hs, Ht, Fst))

#make some array to store the data
dom = array(0,dim=c(2*num.pops*sample.size, num.loci))
codom = array(0,dim=c(2*num.pops*sample.size, maxPloi*num.loci))
undosaged = array(0,dim=c(2*num.pops*sample.size, maxPloi*num.loci))

#loop over loci to make individual data
for(l in 1:num.loci){
	#also loop over populations
	for(pop in 1:num.pops){
		#get allele frequency for this locus and population
		freq1 = freqs[l,pop]

		#draw biallelic data for the first cytotype
		ran1 = rbinom(ploidy1*sample.size,1,freq1)+1
		dim(ran1) = c(sample.size,ploidy1)
		
		#convert to undosaged markers
		vars1 = apply(ran1,1,var)		
		undos1 = array(0,c(sample.size, maxPloi))
		undos1[vars1 == 0,1:ploidy1] = rep(ran1[vars1 == 0,1],ploidy1)	
		undos1[vars1 != 0,1:2] = rep(c(1,2),each=sum(vars1 != 0))

		#now store dominant, codominant and undosaged data
		range = ((pop-1)*2*sample.size+1):((pop-1)*2*sample.size+sample.size)
		dom[range, l] = apply(undos1-1,1,max)
		codom[range, ((l-1)*maxPloi+1):((l-1)*maxPloi+ploidy1)] = ran1
		undosaged[range,((l-1)*maxPloi+1):((l-1)*maxPloi+maxPloi)] = undos1
	
		#draw biallelic data for the second cytotype
		ran2 = rbinom(ploidy2*sample.size,1,freq1)+1
		dim(ran2) = c(sample.size,ploidy2)
		
		#convert to undosaged markers
		vars2 = apply(ran2,1,var)		
		undos2 = array(0,c(sample.size, maxPloi))
		undos2[vars2 == 0,1:ploidy2] = rep(ran2[vars2 == 0,1],ploidy2)	
		undos2[vars2 != 0,1:2] = rep(c(1,2),each=sum(vars2 != 0))

		#now store dominant, codominant and undosaged data
		range = ((pop-1)*2*sample.size+sample.size+1):(pop*2*sample.size)
		dom[range, l] = apply(ran2-1,1,max)
		codom[range, ((l-1)*maxPloi+1):((l-1)*maxPloi+ploidy2)] = ran2
		undosaged[range, ((l-1)*maxPloi+1):((l-1)*maxPloi+maxPloi)] = undos2
	}#populations
		
}#loci

#perform a pca on the dominant data
pca = prcomp(dom)
vars= 100*pca$sdev / sum(pca$sdev)

#make a plot
ploidies = c("haploid", "diploid", "triploid", "tetraploid", "pentaploid", "hexaploid", "heptaploid", "octoploid", "nonaploid", "decaploid", "undecaploid", "dodecaploid", "tridecaploid", "tetradecaploid", "pentadecaploid", "hexadecaploid")
main = sprintf("SNPS turned into AFLPs: %s vs. %s, m=%s", ploidies[ploidy1], ploidies[ploidy2], mig.rate)

plot(pca$x[,1],pca$x[,2], pch=rep(c(rep(1,sample.size), rep(16,sample.size)),num.pops), col=rep(cols[1:num.pops], each=sample.size*2), xlab = sprintf("PCA1 (%.1f%%)", vars[1]), ylab = sprintf("PCA2 (%.1f%%)", vars[2]), font.lab=2, main=main)
legend("topright",c(ploidies[ploidy1], ploidies[ploidy2], sprintf("pop %d", 1:num.pops)), col=c("black", "black", cols[1:num.pops]), pch=c(1,16, rep(95, num.pops)), bg="white")



#WRITE DOMINANT DATA as STRUCTURE file, with first row of zeroes
out = paste(rep(0, num.loci), sep="\t", collapse="\t")
out = paste(out, "\n", sep="")
for(i in 1:(2*num.pops* sample.size)){
	pop = floor((i-1)/(sample.size))+1

	out = paste(out, sprintf("ind%03d\t%d",i , pop), sep="")
	locdata = paste(dom[i,], sep="\t", collapse="\t")
	out = paste(out, locdata, sep="\t")
	out = paste(out, "\n")
	
}
write(out, file=sprintf("struct_dominant_m%s.txt",mig.rate))

#WRITE CODOMINANT DATA as STRUCTURE file, on separate line for every ploi
out=NULL
for(i in 1:(2*num.pops* sample.size)){
	pop = floor((i-1)/(sample.size))+1

	for(p in 1:maxPloi){
		out = paste(out, sprintf("ind%03d\t%d",i , pop), sep="")
		locdata = paste(codom[i,seq(p,maxPloi*num.loci,maxPloi)], sep="\t", collapse="\t")
		out = paste(out, locdata, sep="\t")
		out = paste(out, "\n")
	}
}
write(out, file=sprintf("struct_codominant_m%s.txt",mig.rate))

#WRITE UNDOSAGED DATA as STRUCTURE file, on separate line for every ploi, with first row of zeroes
out = paste(rep(0, num.loci), sep="\t", collapse="\t")
out = paste(out, "\n", sep="")
for(i in 1:(2*num.pops* sample.size)){
	pop = floor((i-1)/(sample.size))+1

	for(p in 1:maxPloi){
		out = paste(out, sprintf("ind%03d\t%d",i , pop), sep="")
		locdata = paste(undosaged[i,seq(p,maxPloi*num.loci,maxPloi)], sep="\t", collapse="\t")
		out = paste(out, locdata, sep="\t")
		out = paste(out, "\n")
	}
}
write(out, file=sprintf("struct_undosaged_m%s.txt",mig.rate))
